
class CGlPrinter
{
public:
	inline CGlPrinter();
	inline ~CGlPrinter();
	inline bool Begin( char *title );
	inline void End();
	inline double GetAspect();
	inline int GetHorzRes();
	inline int GetVertRes();

private:
	PRINTDLG printData;
	HGLRC hrc_printer;
	int cxPage;				// Dimension of printer page (x)
	int cyPage;				// Dimension of printer page (y)
};

inline double CGlPrinter::GetAspect()
{
	return (double)cxPage/(double)cyPage;
}

inline int CGlPrinter::GetHorzRes()
{
	return cxPage;
}

inline int CGlPrinter::GetVertRes()
{
	return cyPage;
}

inline CGlPrinter::CGlPrinter() :
	hrc_printer( 0 )
{
}

inline CGlPrinter::~CGlPrinter()
{
}

inline bool CGlPrinter::Begin( char *title )
{
	// Pixel format for Printer Device context
	static PIXELFORMATDESCRIPTOR pPrintfd = {
		sizeof(PIXELFORMATDESCRIPTOR),			// Size of this structure
		1,                                      // Version of this structure    
		PFD_DRAW_TO_WINDOW |                    // Draw to Window (not to bitmap)
		PFD_SUPPORT_OPENGL |                    // Support OpenGL calls 
		PFD_SUPPORT_GDI |                       // Allow GDI drawing in window too
		PFD_DEPTH_DONTCARE,						// Don't care about depth buffering
		PFD_TYPE_RGBA,                          // RGBA Color mode
		24,                                     // Want 24bit color 
		0,0,0,0,0,0,                            // Not used to select mode
		0,0,                                    // Not used to select mode
		0,0,0,0,0,                              // Not used to select mode
		0,                                      // Size of depth buffer
		0,                                      // Not used to select mode
		0,                                      // Not used to select mode
		0,				                        // Not used to select mode
		0,                                      // Not used to select mode
		0,0,0 };                                // Not used to select mode

	DOCINFO docInfo;        // Document Info
	int nPixelFormat;		// Pixel format requested

	// Get printer information
	memset(&printData,0,sizeof(PRINTDLG));
	printData.lStructSize = sizeof(PRINTDLG);
	printData.hwndOwner = GetForegroundWindow();
	printData.hDevMode = NULL;
	printData.hDevNames = NULL;
	printData.Flags = PD_RETURNDC | PD_ALLPAGES;

	// Display printer dialog
	if(!PrintDlg(&printData))
	{
		printf( "PrintDlg() failed %lx\n", GetLastError() );
		return false;
	}

	// Get the dimensions of the page
	cxPage = GetDeviceCaps(printData.hDC, HORZRES);
	cyPage = GetDeviceCaps(printData.hDC, VERTRES);

	// Initialize DocInfo structure
	docInfo.cbSize = sizeof(DOCINFO);
	docInfo.lpszDocName = title;
	docInfo.lpszOutput = NULL;


	// Choose a pixel format that best matches that described in pfd
	nPixelFormat = ChoosePixelFormat(printData.hDC, &pPrintfd);

	// Watch for no pixel format available for this printer
	if(nPixelFormat == 0)
		{
		// Delete the printer context
		DeleteDC(printData.hDC);

		printf( "ChoosePixelFormat() failed %lx\n", GetLastError() );
		return false;
		}

	// Start the document and page
	StartDoc(printData.hDC, &docInfo);
	StartPage(printData.hDC);
	
	// Set the pixel format for the device context, but watch for failure
	if(!SetPixelFormat(printData.hDC, nPixelFormat, &pPrintfd))
		{
		// Delete the printer context
		DeleteDC(printData.hDC);

		printf( "SetPixelFormat() failed %lx\n", GetLastError() );
		return false;
		}

	// Create the Rendering context and make it current
	hrc_printer = wglCreateContext(printData.hDC);
	wglMakeCurrent(printData.hDC, hrc_printer);

	// Set viewing volume info
	//glViewport(0,0,cxPage/2,cyPage/2);	// Put this in to restrict area of page.
	
	GLfloat nRange = 100.0f;

	// Reset projection matrix stack
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	// Establish clipping volume (left, right, bottom, top, near, far)
	// This keeps the perspective square regardless of window or page size
    if (cyPage <= cxPage) 
	{
		glOrtho (-nRange, nRange, -nRange*cyPage/cxPage, nRange*cyPage/cxPage, -nRange, nRange);
    }
	else 
	{
		glOrtho (-nRange*cxPage/cyPage, nRange*cxPage/cyPage, -nRange, nRange, -nRange, nRange);
	}

	// Reset Model view matrix stack
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	glRotatef(270.0f, 1.0, 0.0, 0.0);

	return true;
}

inline void CGlPrinter::End()
{
	if ( printData.hDC )
	{
		glFinish();

		wglMakeCurrent( NULL, NULL );

		if ( hrc_printer )
		{
			wglDeleteContext( hrc_printer );
		}

		/* EndPage... */
		if ( EndPage( printData.hDC ) <=0 )
		{
			printf( "EndPage() failed\n" );
		}

		/* EndDoc... */
		if ( EndDoc( printData.hDC ) <=0 )
		{
			printf( "EndDoc() failed\n" );
		}

		// Delete the printer context when done with it
		DeleteDC(printData.hDC);
		
		// Restore window rendering context
		//wglMakeCurrent(hDC, hRC);
	}
}

